/*
 *  TOPPERS/DeviceManager
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      DeviceManager
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2006-2009 by Embedded and Real-Time Systems Laboratory
 *            Graduate School of Information Science, Nagoya Univ., JAPAN  
 * 
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 *
 *  @(#) $Id: device.cpp,v 1.6 2003/12/24 07:40:42 takayuki Exp $
 */

#include "stdafx.h"
#include "devicemanager.h"
#include "device.h"

/////////////////////////////////////////////////////////////////////////////
// CDevice


CDevice::CDevice()
{
	DevID = _Module.AddDevice(this);
}

CDevice::~CDevice()
{
	if(DevID != 0)
		_Module.DeleteDevice(this);
}

STDMETHODIMP CDevice::IsValid(BOOL * valid)
{
	*valid = DevID != 0 ? TRUE : FALSE;
	return S_OK;
}

STDMETHODIMP CDevice::SetDeviceName(BSTR devname)
{
	DeviceName = devname;
	return S_OK;
}

STDMETHODIMP CDevice::Map(unsigned long address, unsigned long size)
{
	if( DevID == 0 )
		return E_HANDLE;
	if(_Module.Map(this,Range(address,size)) == false)
		return E_INVALIDARG;
	return S_OK;
}

STDMETHODIMP CDevice::Unmap(unsigned long address)
{
	if( DevID == 0 )
		return E_HANDLE;
	if(_Module.Unmap(this, address) == false)
		return E_INVALIDARG;
	return S_OK;
}

STDMETHODIMP CDevice::RaiseInterrupt(unsigned long inhno)
{
	if(DevID == 0)
		return E_HANDLE;
	if(_Module.KernelList.empty()) //K؂ȃJ[l𒲂ׂׂH
		return E_ACCESSDENIED;
	if(_Module.RaiseInterrupt(inhno) == false)
		return E_INVALIDARG;
	return S_OK;
}

STDMETHODIMP CDevice::CreateSharedMemory(unsigned long address, unsigned long size, BSTR lpName)
{
	if(DevID == 0)
		return E_HANDLE;
	if(_Module.CreateSharedMemory(this, address, size, lpName) == false)
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CDevice::ReleaseSharedMemory(BSTR lpName)
{
	if(DevID == 0)
		return E_HANDLE;
	if(_Module.ReleaseSharedMemory(this, lpName) == false)
		return E_INVALIDARG;
	return S_OK;
}

STDMETHODIMP CDevice::CreateSharedMemoryAnyName(unsigned long address, unsigned long size, BSTR * name)
{
	if(DevID == 0)
		return E_HANDLE;
	if(( *name = ::SysAllocString(_Module.CreateSharedMemoryAnyName(this, address, size)) ) == NULL)
		return E_FAIL;
	return S_OK;
}

/*
STDMETHODIMP CDevice::ReleaseSharedMemory2(unsigned long address)
{
	if(DevID == 0)
		return E_HANDLE;
	if(_Module.ReleaseSharedMemory2(address) == false)
		return E_INVALIDARG;
	return S_OK;
}
*/

STDMETHODIMP CDevice::AcquireLock(unsigned long address, unsigned long size)
{
	bool res;
	if(DevID == 0)
		return E_HANDLE;
	res = _Module.AcquireLock(address, size);
	if(res == false)
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CDevice::ReleaseLock(unsigned long address, unsigned long size)
{
	bool res;
	if(DevID == 0)
		return E_HANDLE;
	res = _Module.ReleaseLock(address, size);
	if(res == false)
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CDevice::CreateMutex(unsigned long number, BSTR * name)
{
	if( ( *name = ::SysAllocString(_Module.CreateMutex(number, this)) ) == NULL )
		return E_FAIL;
	return S_OK;
}

STDMETHODIMP CDevice::CloseMutex(unsigned long number)
{
	_Module.CloseMutex(number, this);
	return S_OK;
}

STDMETHODIMP CDevice::GetMemory(BSTR lpName, unsigned long * address, unsigned long * size)
{
	bool res;
	res = _Module.GetMemory(lpName, address, size);
	if(res == false)
		return E_FAIL;
	return S_OK;
}
